use std::{env, sync::Arc};

use sea_orm::{Database, DatabaseConnection};
use sea_orm_migration::MigratorTrait;

use crate::migrator::Migrator;

pub struct State {
    pub db: DatabaseConnection,
}

impl State {
    pub async fn new() -> Result<Arc<Self>, anyhow::Error> {
        let database_url = env::var("DATABASE_URL").expect("DATABASE_URL must be set");

        let db = Database::connect(database_url).await?;
        Migrator::refresh(&db).await?;

        Ok(Arc::new(State { db }))
    }
}
